package com.persagy.communication.mina.tcp.server;

import com.persagy.communication.entity.Packet;
import com.persagy.constant.Constant;
import lombok.extern.slf4j.Slf4j;
import org.apache.mina.core.service.IoHandlerAdapter;
import org.apache.mina.core.session.IdleStatus;
import org.apache.mina.core.session.IoSession;

import java.net.SocketAddress;
import java.util.Date;

@Slf4j
public class TCPServerIoHandler extends IoHandlerAdapter {
    TCPServerManager manager;

    public TCPServerIoHandler(TCPServerManager manager) {
        this.manager = manager;
    }

    @Override
    public void sessionCreated(IoSession session) throws Exception {
        SocketAddress socketAddress = session.getRemoteAddress();
        String remoteAddress = socketAddress.toString();
        log.info(this.manager.Name() + "\tsessionCreated\t" + remoteAddress);
    }

    @Override
    public void sessionOpened(IoSession session) throws Exception {
        SocketAddress socketAddress = session.getRemoteAddress();
        String remoteAddress = socketAddress.toString();
        this.manager.SetConnectTime(remoteAddress, new Date());
        if (!this.manager.socketMap.containsKey(remoteAddress)) {
            this.manager.socketMap.put(remoteAddress, session);
        }
        log.warn(this.manager.Name() + "\tsessionOpened\t" + remoteAddress);
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionOpened(session.getRemoteAddress());
        }
    }

    @Override
    public void sessionClosed(IoSession session) throws Exception {
        SocketAddress socketAddress = session.getRemoteAddress();
        String remoteAddress = socketAddress.toString();
        this.manager.SetConnectTime(remoteAddress, null);
        this.manager.socketMap.remove(remoteAddress);
        log.warn(this.manager.Name() + "\tsessionClosed\t" + remoteAddress);
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionClosed(session.getRemoteAddress());
        }
        super.sessionClosed(session);
    }

    @Override
    public void exceptionCaught(IoSession session, Throwable cause) throws Exception {
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.exceptionCaught(session.getRemoteAddress(), cause);
        }
        log.error(this.manager.Name() + "\texceptionCaught\t" + session.getRemoteAddress());
        log.error(cause.getMessage(), cause);
    }

    @Override
    public void sessionIdle(IoSession session, IdleStatus status) throws Exception {
        SocketAddress socketAddress = session.getRemoteAddress();
        String remoteAddress = socketAddress.toString();
        log.warn(this.manager.Name() + "\tsessionIdle\t" + remoteAddress);
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionIdle(session.getRemoteAddress());
        }
        session.closeNow();
    }

    @Override
    public void messageReceived(IoSession session, Object message) throws Exception {
        Packet packet = (Packet) message;
        SocketAddress socketAddress = session.getRemoteAddress();
        this.manager.AppendToRece(socketAddress.toString(), packet);
        if(Constant.iotLog){
            log.info( "{}\trece\t{}\t{}",this.manager.Name(),socketAddress.toString(),packet.packetString);
        }

    }


    @Override
    public void messageSent(IoSession session, Object message) throws Exception {
        if(Constant.iotLog){
            Packet packet = (Packet) message;
            SocketAddress socketAddress = session.getRemoteAddress();
            log.info( "{}\tsend\t{}\t{}",this.manager.Name(),socketAddress.toString(),packet.packetString);
        }
    }
}
