package com.persagy.communication.mina.udp.client;

import com.persagy.communication.entity.Packet;
import com.persagy.constant.Constant;
import lombok.extern.slf4j.Slf4j;
import org.apache.mina.core.service.IoHandlerAdapter;
import org.apache.mina.core.session.IdleStatus;
import org.apache.mina.core.session.IoSession;

@Slf4j
public class UDPClientIoHandler extends IoHandlerAdapter {
    UDPClientManager manager;

    public UDPClientIoHandler(UDPClientManager manager) {
        this.manager = manager;
    }

    @Override
    public void sessionCreated(IoSession session) throws Exception {
        log.info(this.manager.Name() + "\tsessionCreated");
    }

    @Override
    public void sessionOpened(IoSession session) throws Exception {
        this.manager.SetSessionClosed(false);
        log.warn(this.manager.Name() + "\tsessionOpened");
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionOpened();
        }
    }

    @Override
    public void sessionClosed(IoSession session) throws Exception {
        this.manager.SetSessionClosed(true);
        log.warn(this.manager.Name() + "\tsessionClosed");
        this.manager.Stop();
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionClosed();
        }
    }

    @Override
    public void exceptionCaught(IoSession session, Throwable cause) throws Exception {
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.exceptionCaught(cause);
        }
    }

    @Override
    public void sessionIdle(IoSession session, IdleStatus status) throws Exception {
        log.warn(this.manager.Name() + "\tsessionIdle");
        if (this.manager.handlerManager != null) {
            this.manager.handlerManager.sessionIdle();
        }
        session.closeNow();
    }

    @Override
    public void messageReceived(IoSession session, Object message) throws Exception {
        Packet packet = (Packet) message;
        this.manager.AppendToRece(packet);
        if(Constant.iotLog){
            log.info("{}\trece\t{}",this.manager.Name(),packet.packetString);
        }
    }

    @Override
    public void messageSent(IoSession session, Object message) throws Exception {
        if(Constant.iotLog){
            Packet packet = (Packet) message;
            log.info( "{}\tsend\t{}",this.manager.Name(),packet.packetString);
        }
    }
}
