package com.persagy.communication.netty.udp.client;

import com.persagy.communication.entity.Packet;
import com.persagy.communication.entity.UDPSendEntity;
import com.persagy.communication.util.*;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.Unpooled;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.DatagramPacket;
import io.netty.channel.socket.nio.NioDatagramChannel;
import lombok.extern.slf4j.Slf4j;

import java.net.DatagramSocket;
import java.net.InetSocketAddress;

@Slf4j
public class NettyUDPClientManager implements IClientManager {
    public final PacketBuffer<Packet> receList = new PacketBuffer<Packet>();
    String ip;
    int port;
    String targetIp;
    int targetPort;
    String encoding;
    String aes_password;
    boolean compress = false;
    int Max_size = 1000;
    boolean sessionClosed = false;
    IClientHandler handlerManager;
    boolean started = false;
    EventLoopGroup group = new NioEventLoopGroup(1);
    Channel channel;
    InetSocketAddress remoteAddress;
    MyEncoder encoder;
    MyDecoder decoder;
    int remark_count = 0;

    public NettyUDPClientManager(String targetIp, int targetPort, String encoding, int Max_size) {
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.Max_size = Max_size;
    }

    public NettyUDPClientManager(String ip, int port, String targetIp, int targetPort, String encoding, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.Max_size = Max_size;
    }

    public NettyUDPClientManager(String targetIp, int targetPort, String encoding, boolean compress, int Max_size) {
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;
    }

    public NettyUDPClientManager(String ip, int port, String targetIp, int targetPort, String encoding,
                                 boolean compress, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.compress = compress;
        this.Max_size = Max_size;
    }

    public NettyUDPClientManager(String ip, int port, String targetIp, int targetPort, String encoding,
                                 String aes_password, boolean compress, int Max_size) {
        this.ip = ip;
        this.port = port;
        this.targetIp = targetIp;
        this.targetPort = targetPort;
        this.encoding = encoding;
        this.aes_password = aes_password;
        this.compress = compress;
        this.Max_size = Max_size;
    }

    @Override
    public void setHandlerManager(IClientHandler handlerManager) {
        this.handlerManager = handlerManager;
    }

    @Override
    public String Name() {
        if (this.ip == null) {
            return "UDP Client " + this.targetIp + ":" + this.targetPort;
        } else {
            return "UDP Client " + this.ip + ":" + this.port;
        }
    }

    @Override
    public synchronized boolean IsSessionClosed() {
        return this.sessionClosed;
    }

    @Override
    public synchronized void SetSessionClosed(boolean value) {
        this.sessionClosed = value;
    }

    @Override
    public synchronized void Start() {
        if (this.started) {
            return;
        }

        try {
            if (this.ip == null) {
                this.ip = "0.0.0.0";
                int tmp_port = 2046;
                DatagramSocket DatagramSocket;
                while (true) {
                    try {
                        DatagramSocket = new DatagramSocket(null);
                        DatagramSocket.bind(new InetSocketAddress(this.ip, tmp_port));
                        break;
                    } catch (Exception e) {
                        tmp_port++;
                    }
                }
                DatagramSocket.close();
                this.port = tmp_port;
            }
            remoteAddress = new InetSocketAddress(this.targetIp, this.targetPort);
            encoder = new MyEncoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, false, false,
                    (byte) '(', (byte) ')', null, null);
            decoder = new MyDecoder(this.ip, this.port, this.encoding, this.aes_password, this.compress, false, false,
                    (byte) '(', (byte) ')', null, null);
            final NettyUDPClientHandler handler = new NettyUDPClientHandler(this);
            Bootstrap b = new Bootstrap();
            b.group(group).channel(NioDatagramChannel.class).option(ChannelOption.SO_BROADCAST, true)
                    .handler(new ChannelInitializer<NioDatagramChannel>() {
                        @Override
                        protected void initChannel(NioDatagramChannel ch) throws Exception {
                            ch.pipeline().addLast(handler);
                        }
                    });

            ChannelFuture cf = b.bind(this.ip, this.port).sync();
            channel = cf.channel();

            this.started = true;
            log.warn(this.Name() + "\tStart");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    @Override
    public void Stop() {
        for (int i = 0; i < this.remark_count; i++) {
            this.receList.offer(new Packet(), Integer.MAX_VALUE);
        }
        if (!this.started) {
            return;
        }

        try {
            group.shutdownGracefully();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    @Override
    public void AppendToSend(Packet packet) {
        try {
            byte[] encoded = this.encoder.encode(packet);
            channel.writeAndFlush(new DatagramPacket(Unpooled.copiedBuffer(encoded), remoteAddress));

            if (this.encoding != null) {
                log.info(this.Name() + "\tsend\t" + packet.packetString);
            } else {
                log.info(this.Name() + "\tsend\t" + HexToByte.byteToHex(packet.packetBytes));
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    public UDPSendEntity PopSend() {
        return null;
    }

    @Override
    public Packet PopRece() {
        Packet entity = this.receList.poll();
        return entity;
    }

    @Override
    public Packet takeRece() throws InterruptedException {
        Packet MyPackage = this.receList.take();
        return MyPackage;
    }

    @Override
    public int ReceBufferSize() {
        return this.receList.BufferSize();
    }

    public synchronized void closeClient(String clientAddress) {
    }

    @Override
    public synchronized void hasRemark(int count) {
        remark_count = count;
    }
}
