package com.persagy.communication.util;

import cn.hutool.core.util.NumberUtil;
import com.persagy.communication.entity.Packet;
import com.persagy.communication.entity.PacketEntity;
import com.persagy.communication.mina.udp.client.UDPClientManager;
import com.persagy.communication.mina.udp.server.UDPServerManager;
import com.persagy.communication.netty.tcp.client.NettyTCPClientManager;
import com.persagy.communication.netty.tcp.server.NettyTCPServerManager;
import com.persagy.util.DateUtils;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
public class ASampleMain {

    public static void main(String[] args) throws Exception {

        int index_args = 0;
        String protocol = args[index_args++];
        String server_or_client = args[index_args++];
        String ip = args[index_args++];
        String port_desc = args[index_args++];
        int cycle_seconds = NumberUtil.parseInt(args[index_args++]);

        int port_from;
        int port_to;
        int index_ = port_desc.indexOf("-");
        if (index_ == -1) {
            port_from = NumberUtil.parseInt(port_desc);
            port_to = NumberUtil.parseInt(port_desc);
        } else {
            port_from = NumberUtil.parseInt(port_desc.substring(0, index_));
            port_to = NumberUtil.parseInt(port_desc.substring(index_ + 1));
        }
        while (true) {
            if ("client".equalsIgnoreCase(server_or_client)) {
                int count_each_port = NumberUtil.parseInt(args[index_args++]);
                int sleep = NumberUtil.parseInt(args[index_args++]);
                List<IClientManager> clientList;
                if ("udp".equalsIgnoreCase(protocol)) {
                    String local_ip = args[index_args++];
                    int local_port_start = NumberUtil.parseInt(args[index_args++]);
                    clientList = Init_UDPClient(ip, port_from, port_to, count_each_port, local_ip, local_port_start);
                } else {
                    clientList = Init_TCPClient(ip, port_from, port_to, count_each_port);
                }
                Process_client(clientList, cycle_seconds, sleep);
            } else if ("ssl_client".equalsIgnoreCase(server_or_client)) {
                int count_each_port = NumberUtil.parseInt(args[index_args++]);
                int sleep = NumberUtil.parseInt(args[index_args++]);
                List<IClientManager> clientList;
                if ("tcp".equalsIgnoreCase(protocol)) {
                    clientList = Init_SSLTCPClient(ip, port_from, port_to, count_each_port);
                    Process_client(clientList, cycle_seconds, sleep);
                }
            } else if ("ssl_server".equalsIgnoreCase(server_or_client)) {
                List<IServerManager> serverList;
                if ("tcp".equalsIgnoreCase(protocol)) {
                    serverList = Init_SSLTCPServer(ip, port_from, port_to);
                    Process_server(serverList, cycle_seconds);
                }
            } else {
                List<IServerManager> serverList;
                if ("udp".equalsIgnoreCase(protocol)) {
                    serverList = Init_UDPServer(ip, port_from, port_to);
                } else {
                    serverList = Init_TCPServer(ip, port_from, port_to);
                }
                Process_server(serverList, cycle_seconds);
            }
        }
    }

    private static void Process_client(List<IClientManager> clientList, int cycle_seconds, int sleep) {
        Date start_time = new Date();
        while (true) {
            try {
                Thread.sleep(sleep);
            } catch (InterruptedException e) {
                log.error(e.getMessage(), e);
            }

            Date curr_time = new Date();
            if (curr_time.getTime() - start_time.getTime() > 1000L * cycle_seconds) {
                break;
            }
            String timeString = DateUtils.formatDate(curr_time, DateUtils.FORMATTER_MILLI_PRETTY);

            try {
                for (IClientManager client : clientList) {
                    client.AppendToSend(new Packet(timeString + "\t" + "I am " + client.Name()));
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        for (IClientManager manager : clientList) {
            manager.Stop();
        }
    }

    private static void Process_server(List<IServerManager> serverList, int cycle_seconds) {
        Date start_time = new Date();
        while (true) {
            try {
                Thread.sleep(1L);
            } catch (InterruptedException e) {
                log.error(e.getMessage(), e);
            }

            Date curr_time = new Date();
            if (curr_time.getTime() - start_time.getTime() > 1000L * cycle_seconds) {
                break;
            }
            String timeString = DateUtils.formatDate(curr_time, DateUtils.FORMATTER_MILLI_PRETTY);

            try {
                for (IServerManager server : serverList) {
                    while (true) {
                        PacketEntity PacketEntity = server.PopRece();
                        if (PacketEntity == null) {
                            break;
                        }

                        server.AppendToSend(PacketEntity.address,
                                new Packet(timeString + "\t" + "I am " + server.Name()));
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        for (IServerManager manager : serverList) {
            manager.Stop();
        }
    }

    private static List<IServerManager> Init_TCPServer(String ip, int port_from, int port_to) throws Exception {
        List<IServerManager> result = new ArrayList<IServerManager>();
        for (int i = port_from; i <= port_to; i++) {
            IServerManager serverManager = new NettyTCPServerManager(ip, i, "utf-8", 1000, (byte) '(', (byte) ')');
            serverManager.Start();
            result.add(serverManager);
        }
        return result;
    }

    private static List<IServerManager> Init_SSLTCPServer(String ip, int port_from, int port_to) throws Exception {
        List<IServerManager> result = new ArrayList<IServerManager>();
        for (int i = port_from; i <= port_to; i++) {

            byte[] ssl_file_km = FileToByte.file2byte(System.getProperty("user.dir") + "/server.jks");
            byte[] ssl_file_tm = FileToByte.file2byte(System.getProperty("user.dir") + "/server.jks");
            String ssl_password_km = "pass_s";
            String ssl_password_tm = "pass_s";
            IServerManager serverManager = new NettyTCPServerManager(ip, i, "utf-8", "TLS", "JKS", "SunX509",
                    ssl_file_km, ssl_password_km, ssl_file_tm, ssl_password_tm, false, null, 1000, (byte) '(',
                    (byte) ')');
            serverManager.Start();
            result.add(serverManager);
        }
        return result;
    }

    private static List<IServerManager> Init_UDPServer(String ip, int port_from, int port_to) throws Exception {
        List<IServerManager> result = new ArrayList<IServerManager>();
        for (int i = port_from; i <= port_to; i++) {
            IServerManager serverManager = new UDPServerManager(ip, i, "utf-8", 1000);
            serverManager.Start();
            result.add(serverManager);
        }
        return result;
    }

    private static List<IClientManager> Init_SSLTCPClient(String ip, int port_from, int port_to, int count_each_port)
            throws Exception {
        List<IClientManager> result = new ArrayList<IClientManager>();
        for (int i = port_from; i <= port_to; i++) {
            for (int ii = 0; ii < count_each_port; ii++) {
                byte[] ssl_file_km = FileToByte.file2byte(System.getProperty("user.dir") + "/client1.jks");
                byte[] ssl_file_tm = FileToByte.file2byte(System.getProperty("user.dir") + "/client1.jks");
                String ssl_password_km = "pass_c";
                String ssl_password_tm = "pass_c";
                IClientManager serverManager = new NettyTCPClientManager(ip, i, "utf-8", "TLS", "JKS", "SunX509",
                        ssl_file_km, ssl_password_km, ssl_file_tm, ssl_password_tm, false, null, 1000, (byte) '(',
                        (byte) ')');
                serverManager.Start();
                result.add(serverManager);
            }
        }
        return result;
    }

    private static List<IClientManager> Init_TCPClient(String ip, int port_from, int port_to, int count_each_port)
            throws Exception {
        List<IClientManager> result = new ArrayList<IClientManager>();
        for (int i = port_from; i <= port_to; i++) {
            for (int ii = 0; ii < count_each_port; ii++) {
                IClientManager serverManager = new NettyTCPClientManager(ip, i, "utf-8", 1000, (byte) '(', (byte) ')');
                serverManager.Start();
                result.add(serverManager);
            }
        }
        return result;
    }

    private static List<IClientManager> Init_UDPClient(String ip, int port_from, int port_to, int count_each_port,
                                                       String local_ip, int local_port_start) throws Exception {
        List<IClientManager> result = new ArrayList<IClientManager>();
        for (int i = port_from; i <= port_to; i++) {
            for (int ii = 0; ii < count_each_port; ii++) {
                int local_port = local_port_start + i * count_each_port + ii;
                IClientManager serverManager = new UDPClientManager(local_ip, local_port, ip, i, "utf-8", 1000);
                serverManager.Start();
                result.add(serverManager);
            }
        }
        return result;
    }

}
