package com.persagy.simulator_gaopin;

import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.MqttTopic;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.communication.entity.Packet;
import com.persagy.communication.mina.tcp.client.TCPClientManager;
import com.persagy.communication.mina.udp.client.UDPClientManager;
import com.persagy.communication.util.IClientManager;
import com.persagy.parser.ParserEntity;
import com.persagy.parser.RecordData;
import com.persagy.util.PacketNumGenerator;
@Slf4j
public class MixClientManager {
	public String mode = "socket";// socket mqtt

	public IClientManager ClientManager;

	public MqttClient MqttClient;
	public Map<String, Map<String, MqttTopic>> topicMapMap = new ConcurrentHashMap<String, Map<String, MqttTopic>>();

	public MixClientManager() {
		this.mode = Constant.socket_or_mqtt;
	}

	public String Name() {
		String result = this.ClientManager.Name();
		return result;
	}

	public void Start() {
		try {
			if (Constant.socket_or_mqtt.equals("socket")) {
				if ("UDP".equalsIgnoreCase(Constant.PROTOCOL)) {
					ClientManager = new UDPClientManager(Constant.ServerIP, Constant.ServerPort, "utf-8", Constant.Compress, Constant.Max_size);
				} else {
					ClientManager = new TCPClientManager(Constant.ServerIP, Constant.ServerPort, "utf-8", Constant.Compress, Constant.Max_size,
							(byte) Constant.Separate_begin.charAt(0), (byte) Constant.Separate_end.charAt(0));
				}
				ClientManager.Start();
			} else if (Constant.socket_or_mqtt.equals("mqtt")) {
				MqttClient = new MqttClient(Constant.mqtt_options.getServerURIs()[0], Constant.mqtt_clientId, new MemoryPersistence());
				MyMqttCallback listener = new MyMqttCallback(this, Constant.mqtt_topic_prefix);
				MqttClient.setCallback(listener);
				MqttClient.connect(Constant.mqtt_options);
				log.warn(Constant.mqtt_options.getServerURIs()[0]);
			}
		} catch (Exception e) {
		}
	}

	public void report(List<ParserEntity> sendList) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
		if (mode.equals("socket")) {
			StringBuffer sb = new StringBuffer();
			for (ParserEntity ParserEntity : sendList) {
				if (sb.length() > 0) {
					sb.append("&");
				}
				String packetString = ParserEntity.building + ";" + ParserEntity.gateway + ";" + "report" + ";"
						+ sdf.format(ParserEntity.dataList.get(0).time) + ";" + ParserEntity.packageId + ";" + ParserEntity.dataList.get(0).meter
						+ ";" + ParserEntity.dataList.size();
				sb.append(packetString);
				for (RecordData RecordData : ParserEntity.dataList) {
					sb.append(";" + RecordData.funcid + ";" + RecordData.value);
				}
			}
			ClientManager.AppendToSend(new Packet(sb.toString()));
		} else if (mode.equals("mqtt")) {
			for (ParserEntity entity : sendList) {
				JSONObject pointset = new JSONObject();
				pointset.put("sequence_no", PacketNumGenerator.Instance().Generate());
				pointset.put("mqtt_client_id", MqttClient.getClientId());
				pointset.put("packet_type", "report");
				JSONArray content = new JSONArray();
				for (int i = 0; i < entity.dataList.size(); i++) {
					RecordData RecordData = entity.dataList.get(i);
					JSONObject contentItem = new JSONObject();
					contentItem.put("meter", RecordData.meter);
					contentItem.put("funcid", RecordData.funcid);
					contentItem.put("time", sdf.format(RecordData.time));
					contentItem.put("value", RecordData.value);
					content.add(contentItem);
				}
				pointset.put("content", content);
				try {
					MqttMessage message = new MqttMessage(pointset.toString().getBytes("UTF-8"));
					MqttTopic topic = this.topicMapMap.get(entity.building).get("report");
					topic.publish(message);
				} catch (Exception e) {
				}
			}
		}
	}

	public void senddownsetack(ParserEntity ParserEntity) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
		if (mode.equals("socket")) {
			for (RecordData RecordData : ParserEntity.dataList) {
				if (RecordData.result.equals("failure_timeout") || RecordData.result.equals("failure_point_not_exist")) {
					continue;
				}

				StringBuffer sb = new StringBuffer();
				String packetString = ParserEntity.building + ";" + ParserEntity.gateway + ";" + "senddownsetack" + ";"
						+ sdf.format(ParserEntity.dataList.get(0).time) + ";" + ParserEntity.packageId;
				sb.append(packetString);
				{
					sb.append(";" + RecordData.meter + ";" + RecordData.funcid + ";" + RecordData.value);
					sb.append(";" + sdf.format(RecordData.endtime));
					sb.append(";" + RecordData.result);
				}
				log.warn("send: " + sb.toString());
				ClientManager.AppendToSend(new Packet(sb.toString()));
			}
		} else if (mode.equals("mqtt")) {
			JSONArray content = new JSONArray();
			for (RecordData RecordData : ParserEntity.dataList) {
				if (RecordData.result.equals("failure_timeout") || RecordData.result.equals("failure_point_not_exist")) {
					continue;
				}

				JSONObject contentItem = new JSONObject();
				contentItem.put("meter", RecordData.meter);
				contentItem.put("funcid", RecordData.funcid);
				contentItem.put("time", sdf.format(RecordData.time));
				contentItem.put("value", RecordData.value);
				contentItem.put("endtime", sdf.format(RecordData.endtime));
				contentItem.put("result", RecordData.result);
				content.add(contentItem);
			}
			if (content.size() > 0) {
				JSONObject pointset = new JSONObject();
				pointset.put("sequence_no", ParserEntity.packageId);
				pointset.put("mqtt_client_id", MqttClient.getClientId());
				pointset.put("packet_type", "pointsetack");
				pointset.put("content", content);
				try {
					MqttMessage message = new MqttMessage(pointset.toString().getBytes("UTF-8"));
					MqttTopic topic = this.topicMapMap.get(ParserEntity.building).get("setup");
					topic.publish(message);
				} catch (Exception e) {
				}
			}
		}
	}
}
