package com.persagy.simulator_gaopin;

import java.io.ByteArrayInputStream;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Handler;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.RandomUtil;
import org.antlr.runtime.ANTLRInputStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.tree.CommonTree;
import org.antlr.runtime.tree.CommonTreeNodeStream;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.entity.ValueObject;
import com.persagy.simulator.MeterSimulatorLexer;
import com.persagy.simulator.MeterSimulatorParser;
import com.persagy.simulator.MeterSimulatorScanner;
import com.persagy.simulator.MeterSimulatorWalker;

public class SimulatorUtil {
	public static final Map<String, Double> accMap = new HashMap<>();
	static DecimalFormat df = new DecimalFormat("#.##");

	public synchronized static Object getValue(EntityGroup group, String meter, int funcid, String data_type, Date curr_time,String key) throws Exception {
		int hash = BKDRHash.getHashCode(meter + "_" + funcid);
		Object dataString;
		if (data_type.equalsIgnoreCase("BOOLEAN") || data_type.equalsIgnoreCase("ENUM")) {
			if (group.lower_limit != null && group.upper_limit != null && group.lower_limit.intValue() >= 0 && group.upper_limit.intValue() <= 1
					&& group.lower_limit.intValue() == group.upper_limit.intValue()) {
				dataString = group.lower_limit.intValue();
			} else {
				JSONArray formula_param = Constant.formula_paramMap.get(data_type);
				int tmp_index = hash % formula_param.size();
				if (tmp_index < 0) {
					tmp_index = 0 - tmp_index;
				}
				JSONObject funcidJSON = formula_param.getJSONObject(tmp_index);
				String formula_name = (String) funcidJSON.get("formula");
				JSONObject params = (JSONObject) funcidJSON.get("params");
				MeterSimulatorWalker walker = Constant.walkerMap.get(formula_name);
				dataString = (int) SimulatorUtil.getData(curr_time, walker, params).data;
			}
		}else if (data_type.equalsIgnoreCase("ACC")) {
			double value = 0;
			if (group.lower_limit != null && group.upper_limit != null) {
				value = accMap.getOrDefault(key, group.lower_limit.doubleValue()) + RandomUtil.randomDouble(group.upper_limit.doubleValue());
			}if (group.lower_limit != null) {
				value = accMap.getOrDefault(key, group.lower_limit.doubleValue()) + RandomUtil.randomDouble(3.0);
			} else {
				value = accMap.getOrDefault(key, 0.0)+ RandomUtil.randomDouble(3.0);
			}
			value = NumberUtil.round(value,2,RoundingMode.FLOOR).doubleValue();
			accMap.put(key,value);
			dataString =value;
		}else {
			JSONArray formula_param = Constant.formula_paramMap.get(data_type);
			int tmp_index = hash % formula_param.size();
			if (tmp_index < 0) {
				tmp_index = 0 - tmp_index;
			}
			JSONObject funcidJSON = formula_param.getJSONObject(tmp_index);
			String formula_name = (String) funcidJSON.get("formula");
			JSONObject params = (JSONObject) funcidJSON.get("params");
			MeterSimulatorWalker walker = Constant.walkerMap.get(formula_name);
			if (group.lower_limit != null && group.upper_limit != null) {
				params.put("b", group.lower_limit);
				params.put("k", group.upper_limit - group.lower_limit);
			}
			dataString = df.format(SimulatorUtil.getData(curr_time, walker, params).data);
		}
		return dataString;
	}

	/**
	 * 检查公式是否正确
	 * 
	 * @param expressions
	 * @return
	 */
	public static String checkSimulator(String expressions) {
		CommonTree t = null;
		CommonTreeNodeStream nodes = null;
		try {
			ByteArrayInputStream in = new ByteArrayInputStream((expressions).getBytes());
			// 词法分析：生成token
			ANTLRInputStream input = new ANTLRInputStream(in);
			MeterSimulatorLexer lexer = new MeterSimulatorLexer(input);
			CommonTokenStream tokens = new CommonTokenStream(lexer);

			// 语法分析：生成抽象语法树
			MeterSimulatorParser parser = new MeterSimulatorParser(tokens);
			MeterSimulatorParser.prog_return r = parser.prog();
			t = (CommonTree) r.getTree();
			nodes = new CommonTreeNodeStream(t);
			nodes.setTokenStream(tokens);
			return "";
		} catch (Exception e) {
			// 如果表达式不符合规则，会抛出异常
			e.printStackTrace();
			return "错误:" + e.getMessage();
		}
	}

	public static MeterSimulatorWalker getWalker(String expressions) throws Exception {
		MeterSimulatorWalker walker = null;
		CommonTree t = null;
		CommonTreeNodeStream nodes = null;
		try {
			ByteArrayInputStream in = new ByteArrayInputStream(expressions.getBytes());
			// 词法分析：生成token
			ANTLRInputStream input = new ANTLRInputStream(in);
			MeterSimulatorLexer lexer = new MeterSimulatorLexer(input);
			CommonTokenStream tokens = new CommonTokenStream(lexer);

			// 语法分析：生成抽象语法树
			MeterSimulatorParser parser = new MeterSimulatorParser(tokens);
			MeterSimulatorParser.prog_return r = parser.prog();
			t = (CommonTree) r.getTree();
			nodes = new CommonTreeNodeStream(t);
			nodes.setTokenStream(tokens);
		} catch (Exception e) {
			// 如果表达式不符合规则，会抛出异常
			e.printStackTrace();
			throw new Exception("公式格式错误:" + expressions);
		}
		// // 扫描变量
		// List<String> bl = new ArrayList<String>();
		// MeterSimulatorScanner scanner = new MeterSimulatorScanner(nodes);
		// scanner.reset();
		// scanner.prog();
		// Iterator<String> keyIter = scanner.varDict.keySet().iterator();
		// while (keyIter.hasNext()) {
		// String key = keyIter.next();
		// if (!"t".equals(key)) {
		// bl.add(key);
		// }
		// }
		// for (int i = 0; i < bl.size(); i++) {
		// String key = bl.get(i);
		// if (!params.containsKey(key)) {
		// throw new Exception("meter params error!");
		// }
		// }

		// 构造解释器
		walker = new MeterSimulatorWalker(nodes);
		return walker;
	}

	public static synchronized DoubleTimeObject getData(Date tmpDate, MeterSimulatorWalker walker, JSONObject params) throws Exception {
		// 设定参数
		walker.clear();
		walker.put("t", tmpDate.getTime());
		Iterator<String> keyIter = params.keySet().iterator();
		while (keyIter.hasNext()) {
			String key = keyIter.next();
			if (!params.containsKey(key)) {
				throw new Exception("meter params error!");
			}
			String value = params.get(key).toString();
			if (value.contains(".")) {
				walker.put(key, Double.parseDouble(params.get(key).toString()));
			} else {
				walker.put(key, Long.parseLong(params.get(key).toString()));
			}
		}
		// 解释器复位并执行
		walker.reset();
		ValueObject resultInner = walker.prog();
		DoubleTimeObject DoubleTimeObject = new DoubleTimeObject();
		DoubleTimeObject.timefrom = tmpDate;
		double datavalue;
		if (resultInner.type == 0) {
			datavalue = resultInner.intValue;
		} else {
			datavalue = resultInner.doubleValue;
		}
		DoubleTimeObject.data = datavalue;

		return DoubleTimeObject;
	}

	public static JSONObject getParams(String expressions) throws Exception {
		CommonTree t = null;
		CommonTreeNodeStream nodes = null;
		try {
			ByteArrayInputStream in = new ByteArrayInputStream(expressions.getBytes());
			// 词法分析：生成token
			ANTLRInputStream input = new ANTLRInputStream(in);
			MeterSimulatorLexer lexer = new MeterSimulatorLexer(input);
			CommonTokenStream tokens = new CommonTokenStream(lexer);

			// 语法分析：生成抽象语法树
			MeterSimulatorParser parser = new MeterSimulatorParser(tokens);
			MeterSimulatorParser.prog_return r = parser.prog();
			t = (CommonTree) r.getTree();
			nodes = new CommonTreeNodeStream(t);
			nodes.setTokenStream(tokens);
		} catch (Exception e) {
			// 如果表达式不符合规则，会抛出异常
			throw new Exception("公式格式错误!");
		}
		// 扫描变量
		JSONObject params = new JSONObject();
		MeterSimulatorScanner scanner = new MeterSimulatorScanner(nodes);
		scanner.reset();
		scanner.prog();
		Iterator<String> keyIter = scanner.varDict.keySet().iterator();
		while (keyIter.hasNext()) {
			String key = keyIter.next();
			if (!"t".equals(key)) {
				params.put(key, "");
			}
		}
		return params;
	}

}
