package com.persagy.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Filter;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

public class ExcelUtil {

	public static String getContentInner(ExcelSheet sheetContent, int col, int row) {
		String result = null;
		if (sheetContent.contentListList != null) {
			if (sheetContent.contentListList.size() > row) {
				List<String> contentList = sheetContent.contentListList.get(row);
				if (contentList.size() > col) {
					result = contentList.get(col);
				}
			}
		}
		return result;
	}

	public static String getContentInner(ExcelSheet sheetContent, String col, String row) {
		int buildingIndex = -1;
		for (int i = 0; i < sheetContent.titleColList.size(); i++) {
			String name = sheetContent.titleColList.get(i);
			if (name.equals(col)) {
				buildingIndex = i;
				break;
			}
		}

		int titleIndex = -1;
		for (int i = 0; i < sheetContent.titleRowList.size(); i++) {
			String name = sheetContent.titleRowList.get(i);
			if (name.equals(row)) {
				titleIndex = i;
				break;
			}
		}

		String content = null;
		if (buildingIndex == -1 || titleIndex == -1) {
		} else {
			content = sheetContent.contentListList.get(titleIndex).get(buildingIndex);
		}

		return content;
	}

	public static void Write(String filepath, Map<String, ExcelSheet> sheetMap) throws Exception {
		Workbook wb;
		if (filepath.endsWith("xlsx")) {
			wb = new XSSFWorkbook();
		} else {
			wb = new HSSFWorkbook();
		}

		String[] names = sheetMap.keySet().toArray(new String[0]);
		Arrays.sort(names);
		for (String name : names) {
			ExcelSheet sheet = sheetMap.get(name);
			Sheet st = wb.createSheet(name);
			int index_row_start = 0;
			if (sheet.titleColList != null && sheet.titleColList.size() > 0) {
				Row row = st.createRow(0);
				for (int i = 0; i < sheet.titleColList.size(); i++) {
					String title = sheet.titleColList.get(i);
					Cell cell = row.createCell(i);
					cell.setCellValue(title);
				}
				index_row_start = 1;
			}
			int index_col_start = 0;
			if (sheet.titleRowList != null && sheet.titleRowList.size() > 0) {
				for (int i = 0; i < sheet.titleRowList.size(); i++) {
					Row row = st.createRow(index_row_start + i);
					String title = sheet.titleRowList.get(i);
					Cell cell = row.createCell(0);
					cell.setCellValue(title);
				}
				index_col_start = 1;
			}
			for (int index_row = 0; index_row < sheet.contentListList.size(); index_row++) {
				List<String> contentList = sheet.contentListList.get(index_row);
				Row row = st.getRow(index_row_start + index_row);
				if (row == null) {
					row = st.createRow(index_row_start + index_row);
				}
				for (int index_col = 0; index_col < contentList.size(); index_col++) {
					String content = contentList.get(index_col);
					Cell cell = row.createCell(index_col_start + index_col);
					cell.setCellValue(content);
				}
			}
		}

		FileOutputStream os = new FileOutputStream(filepath);
		wb.write(os);
		os.close();
	}

	public static Map<String, ExcelSheet> Read(InputStream stream, boolean titleCol) throws Exception {
		Map<String, ExcelSheet> result = new HashMap<String, ExcelSheet>();
		Workbook workbook = WorkbookFactory.create(stream);
		int numbers = workbook.getNumberOfSheets();
		for (int i = 0; i < numbers; i++) {
			Sheet sheet = workbook.getSheetAt(i);
			String name = sheet.getSheetName();
			ExcelSheet ExcelSheet;
			if (titleCol) {
				ExcelSheet = GetSheetContent_titleCol(sheet);
			} else {
				ExcelSheet = GetSheetContent(sheet);
			}
			result.put(name, ExcelSheet);
		}

		return result;
	}

	public static ExcelSheet ReadSheet(String filepath, String sheetName, boolean titleCol) throws Exception {
		Workbook workbook = WorkbookFactory.create(new File(filepath));
		Sheet sheet = workbook.getSheet(sheetName);
		ExcelSheet ExcelSheet;
		if (titleCol) {
			ExcelSheet = GetSheetContent_titleCol(sheet);
		} else {
			ExcelSheet = GetSheetContent(sheet);
		}

		return ExcelSheet;
	}

	public static ExcelSheet GetSheetContent(Sheet sheet) {
		ExcelSheet result = new ExcelSheet();

		int row_from = 0;
		int row_to = 0;

		int row_tmp = row_from + 1;
		while (true) {
			Row row = sheet.getRow(row_tmp);
			if (row == null) {
				break;
			}
			Cell cell = row.getCell(0);
			if (cell == null) {
				break;
			}
			String cellValue = GetCellValue(cell);
			if (cellValue != null && cellValue.trim().length() > 0) {
				row_to = row_tmp;
				result.titleRowList.add(cellValue.trim());
				row_tmp++;
			} else {
				break;
			}
		}

		Row rowBuildings = sheet.getRow(row_from);
		int col_tmp = 1;
		while (true) {
			Cell cell = rowBuildings.getCell(col_tmp);
			if (cell == null) {
				break;
			}
			String cellValue = GetCellValue(cell);
			if (cellValue != null && cellValue.trim().length() > 0) {
				result.titleColList.add(cellValue.trim());
				col_tmp++;
			} else {
				break;
			}
		}

		for (int index_row = row_from + 1; index_row <= row_to; index_row++) {
			Row row = sheet.getRow(index_row);
			List<String> contentList = new ArrayList<String>();
			for (int index_col = 1; index_col < result.titleColList.size() + 1; index_col++) {
				Cell cell = row.getCell(index_col);
				String content = null;
				if (cell != null) {
					String cellValue = GetCellValue(cell);
					if (cellValue != null && cellValue.trim().length() > 0) {
						content = cellValue.trim();
					}
				}
				contentList.add(content);
			}
			result.contentListList.add(contentList);
		}

		return result;
	}

	public static ExcelSheet GetSheetContent_titleCol(Sheet sheet) {
		ExcelSheet result = new ExcelSheet();

		int row_from = 0;
		int row_to = sheet.getLastRowNum();

		Row rowBuildings = sheet.getRow(row_from);
		int index_last = rowBuildings.getLastCellNum();
		for (int col_tmp = 0; col_tmp <= index_last; col_tmp++) {
			Cell cell = rowBuildings.getCell(col_tmp);
			String value = null;
			if (cell != null) {
				String cellValue = GetCellValue(cell);
				if (cellValue != null && cellValue.trim().length() > 0) {
					value = cellValue.trim();
				}
			}
			result.titleColList.add(value);
		}

		for (int index_row = row_from + 1; index_row <= row_to; index_row++) {
			Row row = sheet.getRow(index_row);
			List<String> contentList = new ArrayList<String>();
			if (row != null) {
				for (int index_col = 0; index_col < result.titleColList.size(); index_col++) {
					Cell cell = row.getCell(index_col);
					String content = null;
					if (cell != null) {
						String cellValue = GetCellValue(cell);
						if (cellValue != null && cellValue.trim().length() > 0) {
							content = cellValue.trim();
						}
					}
					contentList.add(content);
				}
			}
			result.contentListList.add(contentList);
		}

		return result;
	}

	static SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

	public static String GetCellValue(Cell cell) {
		if (cell == null) {
			return null;
		}

		String cellText = null;

		try {
			switch (cell.getCellType()) {
			case Cell.CELL_TYPE_NUMERIC:
				short format = cell.getCellStyle().getDataFormat();
				if (isDateType(cell,format)) {
					//format == 14 || format == 22 || format == 31 || format == 57 || format == 58 || format == 176 || format == 177 || format == 178
					Date date = cell.getDateCellValue();
					long floor = date.getTime() / (1000L * 60) * (1000L * 60);
					long ceiling = floor + (1000L * 60);
					Date parseDate;
					if (date.getTime() - floor > ceiling - date.getTime()) {
						parseDate = new Date(ceiling);
					} else {
						parseDate = new Date(floor);
					}
					cellText = sdf.format(parseDate);
				} else {
					cellText = "" + cell.getNumericCellValue();
				}
				break;
			case Cell.CELL_TYPE_STRING:
				cellText = cell.getStringCellValue();
				break;
			case Cell.CELL_TYPE_FORMULA:
				cell.setCellType(Cell.CELL_TYPE_NUMERIC);
				cellText = "" + cell.getNumericCellValue();
				break;
			case Cell.CELL_TYPE_BLANK:
				break;
			case Cell.CELL_TYPE_BOOLEAN:
				break;
			case Cell.CELL_TYPE_ERROR:
				break;
			default:
			}
		} catch (Exception e) {
			// e.printStackTrace();
		}

		return cellText;
	}


	private static boolean isDateType(Cell cell, int formatIndex) {
		// yyyy-MM-dd----- 14
		// yyyy年m月d日---- 31
		// yyyy年m月------- 57
		// m月d日 ---------- 58
		// HH:mm----------- 20
		// h时mm分 -------- 32
		if (formatIndex == 14 || formatIndex == 31 || formatIndex == 57 || formatIndex == 58 || formatIndex == 20 || formatIndex == 32) {
			return true;
		}

		return org.apache.poi.ss.usermodel.DateUtil.isCellDateFormatted(cell);
	}
}
